-- program to test some of the entry widget's functionality
--

import System (getArgs, ExitCode(ExitSuccess), exitWith)
import Gtk    (GtkWindowType(..),
               GtkObjectSignalHandler(..), gtkObjectSignalConnect,
	       GtkBox, gtkBoxPackStartDefaults,
	       GtkHBox, gtkHBoxNew,
	       GtkVBox, gtkVBoxNew,
	       GtkButton, gtkButtonNewWithLabel,
	       GtkButtonSignalHandler(..), gtkButtonSignalConnect,
	       gtkButtonSignalConnectObject,  
	       GtkCheckButton, gtkCheckButtonNewWithLabel,
	       GtkContainer, gtkContainerBorderWidth, gtkContainerAdd,
	       GtkEditableSignalHandler(..), gtkEditableSignalConnect,
	       GtkEntry, gtkEntryGetText, gtkEntrySetVisibility, 
	       gtkEntrySetEditable, gtkEntryNewWithMaxLength, gtkEntrySetText,
	       gtkEntryAppendText, gtkEntrySelectRegion,
	       GtkToggleButton, gtkToggleButtonNewWithLabel,
	       gtkToggleButtonQueryState, gtkToggleButtonSetState,
	       GtkToggleButtonSignalHandler(..), gtkToggleButtonSignalConnect,
	       GtkWidget, gtkWidgetShow, gtkWidgetDestroy, gtkWidgetSetUSize,
	       GtkWidgetSignalHandler(..), gtkWidgetSignalConnect,
	       GtkWindow, gtkWindowNew, gtkWindowSetTitle,
	       gtkInit, gtkMain, gtkMainQuit)


enter     :: GtkEntry -> IO ()
enter ety  = do
	       text <- gtkEntryGetText ety
	       putStr ("Entry contents: " ++ text ++ "\n")

toggleEditable         :: GtkEntry -> GtkCheckButton -> IO ()
toggleEditable ety cbt  = do
			    flag <- gtkToggleButtonQueryState cbt
			    gtkEntrySetEditable ety flag

toggleVisibility         :: GtkEntry -> GtkCheckButton -> IO ()
toggleVisibility ety cbt  = do
			    flag <- gtkToggleButtonQueryState cbt
			    gtkEntrySetVisibility ety flag

main :: IO ()
main  = do
          preArgs <- getArgs
	  _ <- gtkInit preArgs

	  window <- gtkWindowNew GtkWindowToplevel
	  gtkWindowSetTitle window "GTK+ Entry"
	  gtkWidgetSetUSize window 200 100
	  let winDelHdl = GtkWidgetDeleteEventHandler $ \_ _ -> do
								   gtkMainQuit
								   return False
	  gtkWidgetSignalConnect window winDelHdl
				 
	  gtkContainerBorderWidth window 10

	  -- box for the whole thing
	  --
	  bigBox <- gtkVBoxNew False 10
	  gtkContainerAdd window bigBox
	  gtkWidgetShow bigBox

	  entry <- gtkEntryNewWithMaxLength 50
	  let actHdl = GtkEditableActivateHandler enter
	  gtkEditableSignalConnect entry actHdl
	  gtkEntrySetText entry "hello"
	  gtkEntryAppendText entry " world"
	  fullText <- gtkEntryGetText entry
	  gtkEntrySelectRegion entry 0 (length fullText)
	  gtkBoxPackStartDefaults bigBox entry
	  gtkWidgetShow entry

	  -- box for assembling buttons
	  --
	  butBox <- gtkHBoxNew False 10
	  gtkContainerAdd bigBox butBox
	  gtkWidgetShow butBox

	  -- button to toggle editable status
	  --
	  checkEdit <- gtkCheckButtonNewWithLabel " Editable "
	  gtkBoxPackStartDefaults butBox checkEdit
	  gtkToggleButtonSetState checkEdit True -- should be after the handler
						 -- but _call_GC_ doesn't work
	  let editHdl = GtkToggleButtonToggledHandler (toggleEditable entry)
	  gtkToggleButtonSignalConnect checkEdit editHdl
	  gtkWidgetShow checkEdit

	  -- button to toggle visibility status
	  --
	  checkVisi <- gtkCheckButtonNewWithLabel " Visible "
	  gtkBoxPackStartDefaults butBox checkVisi
	  gtkToggleButtonSetState checkVisi True -- should be after the handler
						 -- but _call_GC_ doesn't work
	  let visiHdl = GtkToggleButtonToggledHandler (toggleVisibility entry)
	  gtkWidgetShow checkVisi

	  buttonQuit <- gtkButtonNewWithLabel " Quit "
	  let quitHandler wd = gtkWidgetDestroy wd >> gtkMainQuit
	  gtkButtonSignalConnectObject 
	    buttonQuit (GtkButtonClickedHandler quitHandler) window
	  gtkBoxPackStartDefaults bigBox buttonQuit
	  gtkWidgetShow buttonQuit

	  gtkWidgetShow window

	  gtkMain
	  exitWith ExitSuccess
