-- test for range widgets (preliminary version - without menus)
--

import System (getArgs, ExitCode(ExitSuccess, ExitFailure), exitWith)
import Monad  (when)
import Gtk    (GtkWindowType(..), GtkUpdateType(..), GtkPositionType(..),
               GtkObjectSignalHandler(..), gtkObjectSignalConnect,
	       GtkAdjustment, gtkAdjustmentNew, gtkAdjustmentGetValue,
	       gtkAdjustmentSetPageIncrement, gtkAdjustmentSetPageSize,
	       gtkAdjustmentChanged,
	       GtkAdjustmentSignalHandler(..), gtkAdjustmentSignalConnect,
	       GtkBox, gtkBoxPackStartDefaults,
	       GtkHBox, gtkHBoxNew,
	       GtkVBox, gtkVBoxNew,
	       GtkButton, gtkButtonNewWithLabel,
	       GtkButtonSignalHandler(..), gtkButtonSignalConnect,
	       gtkButtonSignalConnectObject,  
	       GtkCheckButton, gtkCheckButtonNewWithLabel,
	       GtkContainer, gtkContainerBorderWidth, gtkContainerAdd,
	       GtkLabel, gtkLabelNew,
	       GtkRange, gtkRangeSetUpdatePolicy, 
	       GtkScaleClass, GtkScale,
	       GtkHScale, gtkHScaleNew, gtkScaleSetDigits,
	       gtkScaleSetValuePos, gtkScaleSetDrawValue,
	       GtkHScrollbar, gtkHScrollbarNew,
	       GtkHSeparator, gtkHSeparatorNew,
	       GtkVScale, gtkVScaleNew,
	       GtkToggleButton, gtkToggleButtonSetState,
	       gtkToggleButtonQueryState, 
	       GtkToggleButtonSignalHandler(..), gtkToggleButtonSignalConnect,
	       GtkWidget, GtkWidgetFlag(..), gtkWidgetSetFlags,
	       gtkWidgetGrabDefault, gtkWidgetShow, gtkWidgetDestroy,
	       gtkWidgetSetUSize, 
	       GtkWidgetSignalHandler(..), gtkWidgetSignalConnect,
	       GtkWindow, gtkWindowNew, gtkWindowSetTitle,
	       gtkInit, gtkMain, gtkMainQuit)


scaleSetDefaultValues       :: GtkScaleClass scl => scl -> IO ()
scaleSetDefaultValues scale  =
  do
    gtkRangeSetUpdatePolicy scale GtkUpdateContinuous
    gtkScaleSetDigits scale 1
    gtkScaleSetValuePos scale GtkPosTop
    gtkScaleSetDrawValue scale True

createRangeControl :: IO ()
createRangeControl  =
  do
    window <- gtkWindowNew GtkWindowToplevel
    gtkWindowSetTitle window "Range widget test..."
    let winDelHdl = GtkWidgetDeleteEventHandler $ \_ _ -> do
							    gtkMainQuit
							    return False
    gtkWidgetSignalConnect window winDelHdl

    -- box for the whole thing
    --
    bigBox <- gtkVBoxNew False 0
    gtkContainerAdd window bigBox
    gtkWidgetShow bigBox

    -- box for vscale
    --
    topHBox <- gtkHBoxNew False 10
    gtkContainerBorderWidth topHBox 10
    gtkBoxPackStartDefaults bigBox topHBox
    gtkWidgetShow topHBox

    -- value, lower, upper, step_increment, page_increment, page_size 
    -- note that the page_size value only makes a difference for
    -- scrollbar widgets, and the highest value you'll get is actually
    -- (upper - page_size)
    --
    adj1 <- gtkAdjustmentNew 0.0 0.0 101.0 0.1 1.0 1.0
    vscale <- gtkVScaleNew adj1
    scaleSetDefaultValues vscale
    gtkBoxPackStartDefaults topHBox vscale
    gtkWidgetShow vscale

    -- box for hscale and hscollbar
    --
    vbox <- gtkVBoxNew False 10
    gtkContainerBorderWidth vbox 10
    gtkBoxPackStartDefaults topHBox vbox
    gtkWidgetShow vbox

    -- reuse the same adjustment
    --
    hscale <- gtkHScaleNew adj1
    gtkWidgetSetUSize hscale 200 30
    scaleSetDefaultValues hscale
    gtkBoxPackStartDefaults vbox hscale
    gtkWidgetShow hscale

    -- reuse the same adjustment again
    --
    scrollbar <- gtkHScrollbarNew adj1
    --
    -- notice how this causes the scales to always be updated
    -- continuously when the scrollbar is moved
    --
    gtkRangeSetUpdatePolicy scrollbar GtkUpdateContinuous
    gtkBoxPackStartDefaults vbox scrollbar
    gtkWidgetShow scrollbar

    -- box for the checkbutton
    --
    modBox <- gtkHBoxNew False 10
    gtkContainerBorderWidth modBox 10
    gtkBoxPackStartDefaults bigBox modBox
    gtkWidgetShow modBox

    -- a checkbutton to control whether the value is displayed or not
    --
    button <- gtkCheckButtonNewWithLabel "Display value on scale widgets"
    gtkToggleButtonSetState button True
    let cbDrawValue     :: GtkCheckButton -> IO ()
	cbDrawValue btn  = do
			     active <- gtkToggleButtonQueryState btn
			     gtkScaleSetDrawValue hscale active
			     gtkScaleSetDrawValue vscale active
        checkToggledHdl = GtkToggleButtonToggledHandler cbDrawValue
    gtkToggleButtonSignalConnect button checkToggledHdl
    gtkBoxPackStartDefaults modBox button
    gtkWidgetShow button

    -- RESTRICTION: we should have menu here, which determines the position
    --
    gtkScaleSetValuePos hscale GtkPosLeft
    gtkScaleSetValuePos vscale GtkPosLeft

    -- RESTRICTION: we should have another menu here, which determines the
    --		    policy 
    --
    gtkRangeSetUpdatePolicy hscale GtkUpdateDiscontinuous
    gtkRangeSetUpdatePolicy vscale GtkUpdateDiscontinuous

    -- box for the digit scale
    --
    dsBox <- gtkHBoxNew False 10
    gtkContainerBorderWidth dsBox 10
    gtkBoxPackStartDefaults bigBox dsBox
    gtkWidgetShow dsBox

    -- a GtkHScale widget for adjusting the number of digits on the
    -- sample scales
    --
    label <- gtkLabelNew "Scale Digits:"
    gtkBoxPackStartDefaults dsBox label
    gtkWidgetShow label

    adj2 <- gtkAdjustmentNew 1.0 0.0 5.0 1.0 1.0 0.0
    let cbDigitsScale     :: GtkAdjustment -> IO ()
	cbDigitsScale adj  = do
			       val <- gtkAdjustmentGetValue adj
			       gtkScaleSetDigits hscale (round val)
			       gtkScaleSetDigits vscale (round val)
        cbDigitsScaleHdl = GtkAdjustmentValueChangedHandler cbDigitsScale
    gtkAdjustmentSignalConnect adj2 cbDigitsScaleHdl
    scale <- gtkHScaleNew adj2
    gtkScaleSetDigits scale 0
    gtkBoxPackStartDefaults dsBox scale
    gtkWidgetShow scale

    -- box for the page size scale
    --
    psBox <- gtkHBoxNew False 10
    gtkContainerBorderWidth psBox 10
    gtkBoxPackStartDefaults bigBox psBox
    gtkWidgetShow psBox

    --  and, one last GtkHScale widget for adjusting the page size of the
    --  scrollbar
    --
    label <- gtkLabelNew "Scrollbar Page Size:"
    gtkBoxPackStartDefaults psBox label
    gtkWidgetShow label
    
    adj2 <- gtkAdjustmentNew 1.0 1.0 101.0 1.0 1.0 0.0
    let cbPageSize     :: GtkAdjustment -> IO ()
	cbPageSize adj  = do
			    val <- gtkAdjustmentGetValue adj
			    --
			    -- set the page size and page increment size of
			    -- the sample adjustment to the value specified by
			    -- the "Page Size" scale
			    --
			    gtkAdjustmentSetPageSize      adj1 val
			    gtkAdjustmentSetPageIncrement adj1 val
			    --
			    -- now emit the "changed" signal to reconfigure
			    -- all the widgets that are attached to this
			    -- adjustment
			    --
			    gtkAdjustmentChanged adj1
			    return ()
        cbPageSizeHdl = GtkAdjustmentValueChangedHandler cbPageSize
    gtkAdjustmentSignalConnect adj2 cbPageSizeHdl
    scale <- gtkHScaleNew adj2
    gtkScaleSetDigits scale 0
    gtkBoxPackStartDefaults psBox scale
    gtkWidgetShow scale

    separator <- gtkHSeparatorNew
    gtkBoxPackStartDefaults bigBox separator
    gtkWidgetShow separator
    
    qvbox <- gtkVBoxNew False 10
    gtkContainerBorderWidth qvbox 10
    gtkBoxPackStartDefaults bigBox qvbox
    gtkWidgetShow qvbox

    buttonQuit <- gtkButtonNewWithLabel " Quit "
    let quitHandler wd = gtkWidgetDestroy wd >> gtkMainQuit
    gtkButtonSignalConnectObject 
      buttonQuit (GtkButtonClickedHandler quitHandler) window
    gtkBoxPackStartDefaults qvbox buttonQuit
    gtkWidgetSetFlags buttonQuit [GtkCanDefault]
    gtkWidgetGrabDefault buttonQuit
    gtkWidgetShow buttonQuit

    gtkWidgetShow window

main :: IO ()
main  = do
          preArgs <- getArgs
	  args <- gtkInit preArgs
	  createRangeControl
	  gtkMain
	  exitWith ExitSuccess
