-- tests GLib/GDK thread support with Haskell threads
--
-- As of GHC 4.02, the runtime system doesn't use OS threads for realizing
-- non-blocking I/O.  Thus, the `labelCounter' thread doesn't get any
-- opportunity to run while we are in `gtkMain'.
--

import System     (getArgs, ExitCode(ExitSuccess, ExitFailure), exitWith)

import Concurrent (forkIO)

import GLib       (gThreadInit)
import Gdk        (gdkThreadsEnter, gdkThreadsLeave)
import Gtk        (GtkWindowType(..),
               GtkObjectSignalHandler(..), gtkObjectSignalConnect,
	       GtkBox, gtkBoxPackStartDefaults,
	       GtkHBox, gtkHBoxNew,
	       GtkVBox, gtkVBoxNew,
	       GtkButton, gtkButtonNewWithLabel,
	       GtkButtonSignalHandler(..), gtkButtonSignalConnect,
	       gtkButtonSignalConnectObject,  
	       GtkContainer, gtkContainerBorderWidth, gtkContainerAdd,
	       GtkEditableSignalHandler(..), gtkEditableSignalConnect,
	       GtkEntry, gtkEntryGetText, gtkEntrySetVisibility, 
	       gtkEntrySetEditable, gtkEntryNewWithMaxLength, gtkEntrySetText,
	       gtkEntryAppendText, gtkEntrySelectRegion,
	       GtkFileSelection, gtkFileSelectionNew,
	       gtkFileSelectionGetFilename, gtkFileSelectionShowFileopButtons,
	       gtkFileSelectionQueryButtons, gtkFileSelectionSetFilename,
	       GtkLabel, gtkLabelNew, gtkLabelSet,
	       GtkRadioButton, gtkRadioButtonNewWithLabel,
	       gtkRadioButtonNewWithLabelFromWidget, 
	       GtkWidget, gtkWidgetShow, gtkWidgetDestroy, gtkWidgetSetUSize,
	       GtkWidgetSignalHandler(..), gtkWidgetSignalConnect,
	       GtkWindow, gtkWindowNew, gtkWindowSetTitle,
	       gtkInit, gtkMain, gtkMainQuit)


labelCounter         :: GtkLabel -> Int -> IO ()
labelCounter label n  = do
			  gdkThreadsEnter		-- async use of GTK+
			  gtkLabelSet label ("Label #" ++ show n)
			  gdkThreadsLeave		-- finished using GTK+
			  labelCounter label (n + 1)


main :: IO ()
main  = do
          preArgs <- getArgs
	  _ <- gtkInit preArgs

          gThreadInit		-- this has to precede any `gdkThreadsXX' call

	  window <- gtkWindowNew GtkWindowToplevel
	  gtkWindowSetTitle window "GTK+ Thread Test"
	  gtkWidgetSetUSize window 180 90
	  let winDelHdl = GtkWidgetDeleteEventHandler $ \_ _ -> do
								   gtkMainQuit
								   return False
	  gtkWidgetSignalConnect window winDelHdl
	  gtkContainerBorderWidth window 10

	  label <- gtkLabelNew "Initial label"
	  gtkContainerAdd window label
	  gtkWidgetShow label

	  gtkWidgetShow window

	  -- from here on, we are multi-threaded
	  --
	  forkIO (labelCounter label 0)

	  gtkMain

	  exitWith ExitSuccess
